"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PulumiAdapter = void 0;
const fs_extra_1 = require("fs-extra");
const crypto_1 = require("crypto");
const path_1 = require("path");
const base_1 = require("@plutolang/base");
const automation_1 = require("@pulumi/pulumi/automation");
const utils_1 = require("./utils");
const STATE_FILE_NAME = "pulumi-state.json";
class PulumiAdapter extends base_1.core.Adapter {
    workDir;
    passphrase;
    status;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    name = require((0, path_1.join)(__dirname, "../package.json")).name;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    version = require((0, path_1.join)(__dirname, "../package.json")).version;
    constructor(args) {
        if (!(0, path_1.isAbsolute)(args.entrypoint)) {
            throw new Error("The entrypoint provided to the adapter must be an absolute path.");
        }
        super(args);
        this.workDir = (0, path_1.join)(this.stateDir, "pulumi");
        (0, fs_extra_1.ensureDirSync)(this.workDir);
        const state = this.load();
        if (state) {
            this.passphrase = state.passphrase;
            this.status = state.status;
        }
        else {
            this.passphrase = (0, crypto_1.randomUUID)();
            this.status = "undeployed";
            this.dump();
        }
    }
    async state() {
        try {
            const pulumiStack = await this.createPulumiStack();
            const stackState = await pulumiStack.workspace.stack();
            if (!stackState) {
                throw new Error("Cannot find the target pulumi stack. Have you already deployed it?");
            }
            await pulumiStack.refresh();
            const result = await pulumiStack.exportStack();
            const instances = [];
            const resources = result.deployment.resources ?? [];
            for (const resource of resources) {
                const instance = {
                    id: resource["urn"],
                    name: resource["urn"].split("::").pop(),
                    type: resource["type"],
                    status: base_1.core.ResourceInstanceStatus.Deployed,
                    parent: resource["parent"],
                };
                instances.push(instance);
            }
            return { instances };
        }
        catch (e) {
            if (e instanceof Error) {
                throw new Error("Met error during run 'pulumi state', " + e.message);
            }
            else {
                throw new Error("Met error during run 'pulumi state', " + e);
            }
        }
    }
    async deploy(opts = {}) {
        try {
            const pulumiStack = await this.createPulumiStack();
            if (this.status === "updating" ||
                (0, utils_1.updateInProgress)(this.project, this.stack.name, this.workDir)) {
                if (opts.force) {
                    await pulumiStack.cancel();
                }
                else {
                    // If the force option is not set, we will throw an error.
                    throw new Error("This stack is currently being updated. If you want to update forcefully, you can use the --force option.");
                }
            }
            this.status = "updating";
            const result = await pulumiStack.up();
            this.status = "deployed";
            return { outputs: result.outputs["default"]?.value ?? {} };
        }
        catch (e) {
            if (process.env.DEBUG) {
                console.error(e);
            }
            if (e instanceof automation_1.CommandError) {
                throw extractError(e);
            }
            else {
                throw e;
            }
        }
        finally {
            this.dump();
        }
    }
    async destroy(opts = {}) {
        try {
            const pulumiStack = await this.createPulumiStack();
            if (this.status === "updating" ||
                (0, utils_1.updateInProgress)(this.project, this.stack.name, this.workDir)) {
                if (opts.force) {
                    await pulumiStack.cancel();
                }
                else {
                    // If the force option is not set, we will throw an error.
                    throw new Error("This stack is currently being updated. If you want to update forcefully, you can use the --force option.");
                }
            }
            await pulumiStack.refresh();
            this.status = "updating";
            await pulumiStack.destroy();
            this.status = "undeployed";
            await pulumiStack.workspace.removeStack(this.stack.name);
        }
        catch (e) {
            if (e instanceof Error) {
                throw new Error("Met error during run 'pulumi destroy', " + e.message);
            }
            else {
                throw new Error("Met error during run 'pulumi destroy', " + e);
            }
        }
        finally {
            this.dump();
        }
    }
    dump() {
        const stateFile = (0, path_1.resolve)(this.workDir, STATE_FILE_NAME);
        const state = {
            passphrase: this.passphrase,
            status: this.status,
        };
        (0, fs_extra_1.writeFileSync)(stateFile, JSON.stringify(state), "utf-8");
    }
    load() {
        const stateFile = (0, path_1.resolve)(this.workDir, STATE_FILE_NAME);
        if ((0, fs_extra_1.existsSync)(stateFile) === false) {
            return;
        }
        const stateData = (0, fs_extra_1.readFileSync)(stateFile, "utf-8");
        const state = JSON.parse(stateData);
        return state;
    }
    async createPulumiStack() {
        const envs = {
            PLUTO_PROJECT_NAME: this.project,
            PLUTO_STACK_NAME: this.stack.name,
            PLUTO_LANGUAGE_TYPE: this.language,
            PLUTO_PLATFORM_TYPE: this.stack.platformType,
            PLUTO_PROVISION_TYPE: this.stack.provisionType,
            PULUMI_CONFIG_PASSPHRASE: this.passphrase,
            WORK_DIR: this.stateDir,
        };
        const pulumiStack = await automation_1.LocalWorkspace.createOrSelectStack({
            stackName: this.stack.name,
            workDir: this.workDir,
        }, {
            workDir: this.workDir,
            envVars: envs,
            projectSettings: {
                runtime: "nodejs",
                name: this.project,
                main: this.entrypoint,
                backend: { url: "file://" + this.workDir },
            },
        });
        for (const key of Object.keys(envs))
            process.env[key] = envs[key];
        const pulumiConfig = await (0, utils_1.genPulumiConfig)(this.stack);
        pulumiConfig["pluto:projectRoot"] = { value: process.cwd() };
        await pulumiStack.setAllConfig(pulumiConfig);
        return pulumiStack;
    }
}
exports.PulumiAdapter = PulumiAdapter;
class PulumiError extends Error {
    constructor(message, stack) {
        stack = stack.replace(/ {4,}/g, "    ");
        super(message);
        this.stack = `${message}\n${stack}`;
    }
}
function extractError(error) {
    const errMsg = error.commandResult.stderr;
    const regex = /Error: ([\s\S]*?)\n([\s\S]*?)(?=\n\n)/;
    const match = errMsg.match(regex);
    if (match) {
        const message = match[1];
        const stack = match[2];
        return new PulumiError(message, stack);
    }
    else {
        const errMsg = error.commandResult.err.shortMessage;
        return new PulumiError(errMsg, "");
    }
}
/* The CommandError example:
{
  "commandResult": {
    "stdout": "",
    "stderr": "Command failed with exit code 255: pulumi up --yes --skip-preview --exec-kind auto.local --stack ali --non-interactive\nUpdating (ali):\n\n    pulumi:pulumi:Stack hello-pluto-ali running \n    pulumi:pulumi:Stack hello-pluto-ali running warning: Could not find entry point '/pluto/project/dist/index.js' specified in package.json; using '/pluto/project/.pluto/ali/generated/pulumi.js' instead\n    pulumi:pulumi:Stack hello-pluto-ali running error: Running program '/pluto/project/.pluto/ali/generated/pulumi.js' failed with an unhandled exception:\n    pulumi:pulumi:Stack hello-pluto-ali running (node:40896) PromiseRejectionHandledWarning: Promise rejection was handled asynchronously (rejection id: 6)\n    pulumi:pulumi:Stack hello-pluto-ali running (Use `node --trace-warnings ...` to show where the warning was created)\n    pulumi:pulumi:Stack hello-pluto-ali **failed** 1 error; 1 warning; 2 messages\nDiagnostics:\n  pulumi:pulumi:Stack (hello-pluto-ali):\n    warning: Could not find entry point '/pluto/project/dist/index.js' specified in package.json; using '/pluto/project/.pluto/ali/generated/pulumi.js' instead\n    error: Running program '/pluto/project/.pluto/ali/generated/pulumi.js' failed with an unhandled exception:\n    Error: The implementation class for the resource type '@plutolang/pluto.KVStore', intended for the 'Pulumi' provisioning engine on the 'AliCloud' platform, cannot be found.\n        at ImplClassMap.loadImplClassOrThrow (/pluto/packages/pluto-infra/src/utils/impl-class-map.ts:60:13)\n        at ImplClassMap.createInstanceOrThrow (/pluto/packages/pluto-infra/src/utils/impl-class-map.ts:81:34)\n        at Function.createInstance (/pluto/packages/pluto-infra/src/kvstore.ts:46:25)\n        at \u001b[90m/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:45:62\n        at step \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:33:23\u001b[90m)\u001b[39m\n        at Object.next \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:14:53\u001b[90m)\u001b[39m\n        at fulfilled \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:5:58\u001b[90m)\u001b[39m\n\n    (node:40896) PromiseRejectionHandledWarning: Promise rejection was handled asynchronously (rejection id: 6)\n    (Use `node --trace-warnings ...` to show where the warning was created)\n\nResources:\n    1 unchanged\n\nDuration: 1s\n",
    "code": -2,
    "err": {
      "shortMessage": "Command failed with exit code 255: pulumi up --yes --skip-preview --exec-kind auto.local --stack ali --non-interactive",
      "command": "pulumi up --yes --skip-preview --exec-kind auto.local --stack ali --non-interactive",
      "escapedCommand": "pulumi up --yes --skip-preview --exec-kind auto.local --stack ali --non-interactive",
      "exitCode": 255,
      "stdout": "Updating (ali):\n\n    pulumi:pulumi:Stack hello-pluto-ali running \n    pulumi:pulumi:Stack hello-pluto-ali running warning: Could not find entry point '/pluto/project/dist/index.js' specified in package.json; using '/pluto/project/.pluto/ali/generated/pulumi.js' instead\n    pulumi:pulumi:Stack hello-pluto-ali running error: Running program '/pluto/project/.pluto/ali/generated/pulumi.js' failed with an unhandled exception:\n    pulumi:pulumi:Stack hello-pluto-ali running (node:40896) PromiseRejectionHandledWarning: Promise rejection was handled asynchronously (rejection id: 6)\n    pulumi:pulumi:Stack hello-pluto-ali running (Use `node --trace-warnings ...` to show where the warning was created)\n    pulumi:pulumi:Stack hello-pluto-ali **failed** 1 error; 1 warning; 2 messages\nDiagnostics:\n  pulumi:pulumi:Stack (hello-pluto-ali):\n    warning: Could not find entry point '/pluto/project/dist/index.js' specified in package.json; using '/pluto/project/.pluto/ali/generated/pulumi.js' instead\n    error: Running program '/pluto/project/.pluto/ali/generated/pulumi.js' failed with an unhandled exception:\n    Error: The implementation class for the resource type '@plutolang/pluto.KVStore', intended for the 'Pulumi' provisioning engine on the 'AliCloud' platform, cannot be found.\n        at ImplClassMap.loadImplClassOrThrow (/pluto/packages/pluto-infra/src/utils/impl-class-map.ts:60:13)\n        at ImplClassMap.createInstanceOrThrow (/pluto/packages/pluto-infra/src/utils/impl-class-map.ts:81:34)\n        at Function.createInstance (/pluto/packages/pluto-infra/src/kvstore.ts:46:25)\n        at \u001b[90m/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:45:62\n        at step \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:33:23\u001b[90m)\u001b[39m\n        at Object.next \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:14:53\u001b[90m)\u001b[39m\n        at fulfilled \u001b[90m(/pluto/project/.pluto/ali/generated/\u001b[39mpulumi.js:5:58\u001b[90m)\u001b[39m\n\n    (node:40896) PromiseRejectionHandledWarning: Promise rejection was handled asynchronously (rejection id: 6)\n    (Use `node --trace-warnings ...` to show where the warning was created)\n\nResources:\n    1 unchanged\n\nDuration: 1s\n",
      "stderr": "",
      "failed": true,
      "timedOut": false,
      "isCanceled": false,
      "killed": false
    }
  },
  "name": "CommandError"
}
*/
//# sourceMappingURL=pulumi.js.map